@@ -4,8 +4,11 @@ use glob::glob;
 use pfc_tester::errors::TerraRustTestingError;
 use pfc_tester::errors::TerraRustTestingError::ExecResponseFail;
 use pfc_tester::{NAME, VERSION};
+use rand::distributions::Alphanumeric;
+use rand::Rng;
 use secp256k1::Secp256k1;
 use serde::{Deserialize, Serialize};
+use std::collections::HashMap;
 use terra_rust_api::bank::MsgSend;
 use terra_rust_api::core_types::Coin;
 use terra_rust_api::{LCDResult, MsgExecuteContract, PrivateKey, Terra};
@@ -61,6 +64,10 @@ pub struct Command {
 }
 #[derive(Deserialize, Serialize, Debug)]
 pub struct CommandFile {
+    /// generate a random string of 10 a-zA-Z letters which will be constant for the life of the run. E:RAND_FILE_1 -> n
+    pub random_per_file: Option<u64>,
+    /// generate a random string of 10 a-zA-Z letters which will be constant for the life of the command. E:RAND_CMD_1 -> n
+    pub random_per_command: Option<u64>,
     pub commands: Vec<Command>,
 }
 /// Run a query, and validate the response matches
@@ -74,6 +81,7 @@ async fn run_query<'a, C: secp256k1::Signing + secp256k1::Context>(
     secp: &Secp256k1<C>,
     wallet: Option<Wallet<'a>>,
     seed: Option<&str>,
+    command_vars: HashMap<String, String>,
 ) -> Result<(), TerraRustTestingError> {
     let query_str = cli_helpers::expand_block(
         &serde_json::to_string(&query)?,
@@ -81,6 +89,8 @@ async fn run_query<'a, C: secp256k1::Signing + secp256k1::Context>(
         secp,
         wallet.clone(),
         seed,
+        Some(command_vars.clone()),
+        true,
     )?;
     let response_str = cli_helpers::expand_block(
         &serde_json::to_string(&response)?,
@@ -88,6 +98,8 @@ async fn run_query<'a, C: secp256k1::Signing + secp256k1::Context>(
         secp,
         wallet,
         seed,
+        Some(command_vars),
+        true,
     )?;
     let response_returned = terra
         .wasm()
@@ -129,6 +141,7 @@ async fn run_exec<'a, C: secp256k1::Signing + secp256k1::Context>(
     retries: usize,
     should_error: Option<bool>,
     error_message: Option<String>,
+    command_vars: HashMap<String, String>,
 ) -> Result<(), TerraRustTestingError> {
     let sender = private_key.public_key(secp).account()?;
     let action_str = cli_helpers::expand_block(
@@ -137,7 +150,10 @@ async fn run_exec<'a, C: secp256k1::Signing + secp256k1::Context>(
         secp,
         wallet.clone(),
         seed,
+        Some(command_vars.clone()),
+        true,
     )?;
+
     let coin_vec = if let Some(coin_str) = coins {
         Coin::parse_coins(&coin_str)?
     } else {
@@ -179,10 +195,19 @@ async fn run_exec<'a, C: secp256k1::Signing + secp256k1::Context>(
                         &response.event_type.clone().unwrap_or_else(|| "-".into());
                     let response_attr_key =
                         &response.attribute_key.clone().unwrap_or_else(|| "-".into());
-                    let response_attr_value = &response
-                        .attribute_value
-                        .clone()
-                        .unwrap_or_else(|| "-".into());
+                    let response_attr_value = cli_helpers::expand_block(
+                        &response
+                            .attribute_value
+                            .clone()
+                            .unwrap_or_else(|| "-".into()),
+                        Some(sender.clone()),
+                        secp,
+                        wallet.clone(),
+                        seed,
+                        Some(command_vars.clone()),
+                        true,
+                    )?;
+
                     if let Some(logs) = &tx.tx_response.logs {
                         for log_entry in logs {
                             if response.msg_index.is_none()
@@ -220,15 +245,17 @@ async fn run_exec<'a, C: secp256k1::Signing + secp256k1::Context>(
                             contract.into(),
                             response_event_type.to_string(),
                             response_attr_key.to_string(),
-                            response_attr_value.to_string(),
+                            response_attr_value,
+                            hash,
                         ));
                     }
                 }
             }
             log::info!(
-                "OK: Exec {} {}",
+                "OK: Exec {} {} {}",
                 comment.unwrap_or_else(|| "-".into()),
-                contract
+                contract,
+                hash
             );
         }
         Err(err_response) => {
@@ -277,9 +304,18 @@ async fn run_send<'a, C: secp256k1::Signing + secp256k1::Context>(
     seed: Option<&str>,
     sleep: u64,
     retries: usize,
+    command_vars: HashMap<String, String>,
 ) -> Result<(), TerraRustTestingError> {
     let sender = private_key.public_key(secp).account()?;
-    let send_to = cli_helpers::expand_block(&to, Some(sender.clone()), secp, Some(wallet), seed)?;
+    let send_to = cli_helpers::expand_block(
+        &to,
+        Some(sender.clone()),
+        secp,
+        Some(wallet),
+        seed,
+        Some(command_vars),
+        true,
+    )?;
 
     let coin_vec = Coin::parse_coins(&coins)?;
 
@@ -323,7 +359,24 @@ async fn run_command_file<'a, C: secp256k1::Context + secp256k1::Signing>(
     sleep: u64,
     retries: usize,
 ) -> Result<(), TerraRustTestingError> {
+    let mut rng = rand::thread_rng();
+    let mut file_vars: HashMap<String, String> = Default::default();
+
+    if let Some(rand_per_file) = command_file.random_per_file {
+        for i in 1..rand_per_file + 1 {
+            let word: String = (0..10).map(|_| rng.sample(Alphanumeric) as char).collect();
+            file_vars.insert(format!("RAND_FILE_{}", i), word);
+        }
+    }
+
     for command in command_file.commands {
+        let mut command_vars: HashMap<String, String> = file_vars.clone();
+        if let Some(rand_per_command) = command_file.random_per_command {
+            for i in 1..rand_per_command + 1 {
+                let word: String = (0..10).map(|_| rng.sample(Alphanumeric) as char).collect();
+                command_vars.insert(format!("RAND_CMD_{}", i), word);
+            }
+        }
         let sender = if let Some(key_name) = command.sender {
             wallet.get_private_key(secp, &key_name, None)?
         } else {
@@ -341,6 +394,7 @@ async fn run_command_file<'a, C: secp256k1::Context + secp256k1::Signing>(
                     secp,
                     Some(wallet.clone()),
                     seed,
+                    command_vars,
                 )
                 .await?;
             }
@@ -363,6 +417,7 @@ async fn run_command_file<'a, C: secp256k1::Context + secp256k1::Signing>(
                     retries,
                     exec.error,
                     exec.error_message,
+                    command_vars,
                 )
                 .await?;
             }
@@ -378,6 +433,7 @@ async fn run_command_file<'a, C: secp256k1::Context + secp256k1::Signing>(
                     seed,
                     sleep,
                     retries,
+                    command_vars,
                 )
                 .await?;
             }
@@ -388,7 +444,6 @@ async fn run_command_file<'a, C: secp256k1::Context + secp256k1::Signing>(
 async fn run_it(matches: &ArgMatches) -> Result<(), TerraRustTestingError> {
     dotenv::from_filename("code_id.env").ok();
     dotenv::from_filename("contracts.env").ok();
-    let resource_directory = cli_helpers::get_arg_value(matches, "test_directory")?;
     let sleep = cli_helpers::get_arg_value(matches, "sleep")?.parse::<u64>()?;
     let retries = cli_helpers::get_arg_value(matches, "retries")?.parse::<usize>()?;
 
@@ -397,8 +452,15 @@ async fn run_it(matches: &ArgMatches) -> Result<(), TerraRustTestingError> {
     let from_key = cli_helpers::get_private_key(&secp, matches)?;
     let wallet = cli_helpers::wallet_from_args(matches)?;
     let seed = cli_helpers::seed_from_args(matches);
-
-    for test_file in glob(&format!("{}/*.test.json", resource_directory))? {
+    let glob_pattern = if let Some(dir) = matches.value_of("test-directory") {
+        format!("{}/*.test.json", dir)
+    } else if let Some(file) = matches.value_of("test-file") {
+        file.to_string()
+    } else {
+        panic!("Either pass in a test-directory or a test-file");
+    };
+    //let resource_directory = cli_helpers::get_arg_value(matches, "test-directory")?;
+    for test_file in glob(&glob_pattern)? {
         let file = test_file?;
         log::info!("{}", file.display());
         // sender is NONE here. will expand later
@@ -408,9 +470,12 @@ async fn run_it(matches: &ArgMatches) -> Result<(), TerraRustTestingError> {
             &secp,
             Some(wallet.clone()),
             seed,
+            None,
+            false,
         )?;
 
         let command_file: CommandFile = serde_json::from_value(expanded_json)?;
+
         run_command_file(
             &terra,
             &secp,
@@ -427,12 +492,16 @@ async fn run_it(matches: &ArgMatches) -> Result<(), TerraRustTestingError> {
 }
 async fn run() -> anyhow::Result<()> {
     let app = cli_helpers::gen_cli("PFC-Replayer", "pfc-replayer").args(&[
-        Arg::new("test_directory")
-            .long("test_directory")
+        Arg::new("test-directory")
+            .long("test-directory")
             .takes_value(true)
-            .value_name("test_directory")
-            .default_value("./resources")
+            .value_name("test-directory")
             .help("directory where test (*.test.json) JSON resides"),
+        Arg::new("test-file")
+            .long("test-file")
+            .takes_value(true)
+            .value_name("test-file")
+            .help("test file to replay"),
         Arg::new("retries")
             .long("retries")
             .takes_value(true)
